from typing import Dict, List

from unittestai import MagicFunction


class State:
    orig_context: str  # all the files at the beginning of the search
    context: str  # the orig_context concatenated with the current implementations from this state
    prompt: str
    ai_output: str
    impls: Dict[str, str]  # For each function, the source-code implementation: func_name -> code implementation.
    mfs: List[MagicFunction]  # the magic functions with their original definitions and "current" implementations
    tests: str  # the source code of the test class
    errors: List[str]  # the errors we got from the tests
    score: float  # the score for this solution
    children: List['State']
    temperature: float = None
    count = None

    def __init__(self):
        self.orig_context: str = ''
        self.context: str = None
        self.prompt: str = None
        self.ai_output: str = None
        self.impls: Dict[str, str] = {}
        self.mfs: List[MagicFunction] = None
        self.tests: str = None
        self.errors: List[str] = []
        self.score: float = None
        self.children: List['State'] = []
        self.temperature: float = None
        self.count = None

    def build_context(self):
        context = self.orig_context
        for mf in self.mfs:
            context += (mf.impl if mf.impl is not None else mf.clean_orig_code) + '\n'
        return context

    def __repr__(self):
        # return f'#{self.count}<{self.score}, {len(self.errors)}, {self.temperature:.3f}>'
        return f'State(id={self.count}, score={self.score:.2f}, errors={len(self.errors)}, temp={self.temperature:.3f})'

    def to_dict(self):
        return {
            'count': self.count,
            'context': self.context,
            'prompt': self.prompt,
            'ai_output': self.ai_output,
            'impls': self.impls,
            'mfs': [mf.to_dict() for mf in self.mfs],
            'tests': self.tests,
            'errors': self.errors,
            'score': self.score,
            'temperature': self.temperature,
            'children': [child.to_dict() for child in self.children]
        }
